@@ -10,13 +10,15 @@
   Copyright (c) 2015 Bronislav Robenek <brona@robenek.me>
 """
 
-from iproute2mac import *
 import ipaddress
 import os
 import re
 import socket
 import subprocess
 import sys
+from operator import itemgetter
+
+from iproute2mac import *
 
 
 # Decode ifconfig output
@@ -91,18 +93,23 @@ def parse_ifconfig(res, af, address):
     if count > 1:
         links.append(link)
 
-    return links
+    return sorted(links, key=itemgetter("ifindex"))
 
 
 def link_addr_show(
     argv, af, json_print, pretty_json, color, address, brief, oneline
 ):
+    param = ""
+    if "up" in argv:
+        argv.remove("up")
+        param = "-u "
+
     if len(argv) > 0 and argv[0] == "dev":
         argv.pop(0)
     if len(argv) > 0:
-        param = argv[0]
+        param += argv[0]
     else:
-        param = "-a"
+        param += "-a"
 
     output_separator = "\\" if oneline else "\n"
 
@@ -243,7 +250,7 @@ def do_help(
 
 
 def do_help_route():
-    perror("Usage: ip route list")
+    perror("Usage: ip route list [[exact] PREFIX]")
     perror("       ip route get ADDRESS")
     perror("       ip route { add | del | replace } ROUTE")
     perror("       ip route flush cache")
@@ -257,13 +264,13 @@ def do_help_route():
 
 
 def do_help_addr():
-    perror("Usage: ip addr show [ dev STRING ]")
+    perror("Usage: ip addr show [ dev STRING ] [up]")
     perror("       ip addr { add | del } PREFIX dev STRING")
     exit(255)
 
 
 def do_help_link():
-    perror("Usage: ip link show [ DEVICE ]")
+    perror("Usage: ip link show [ DEVICE ] [up]")
     perror("       ip link set dev DEVICE")
     perror("                [ { up | down } ]")
     perror("                [ address { LLADDR | factory | random } ]")
@@ -281,9 +288,10 @@ def do_help_neigh():
 @help_msg(do_help_route)
 def do_route(argv, af, json_print, pretty_json, color, brief, oneline):
     if not argv or (
-        any_startswith(["show", "lst", "list"], argv[0]) and len(argv) == 1
+        any_startswith(["show", "lst", "list"], argv[0]) and len(argv) <= 3
     ):
-        return do_route_list(af, json_print, pretty_json, color)
+        argv.pop(0)
+        return do_route_list(argv, af, json_print, pretty_json, color)
     elif "get".startswith(argv[0]) and len(argv) == 2:
         argv.pop(0)
         return do_route_get(argv, af, json_print, pretty_json, color)
@@ -304,7 +312,18 @@ def do_route(argv, af, json_print, pretty_json, color, brief, oneline):
     return True
 
 
-def do_route_list(af, json_print, pretty_json, color):
+def do_route_list(argv, af, json_print, pretty_json, color):
+    # argv can have SELECTOR = [[exact] PREFIX]
+    argc = len(argv)
+    if argc == 0:
+        exact = ""
+    elif argc == 1:
+        exact = argv[0]
+    elif argc == 2 and argv[0] == "exact":
+        exact = argv[1]
+    else:
+        return False
+
     # ip route prints IPv6 or IPv4, never both
     inet = "inet6" if af == 6 else "inet"
     status, res = subprocess.getstatusoutput(
@@ -343,6 +362,9 @@ def do_route_list(af, json_print, pretty_json, color):
                 {"dst": target, "gateway": gw, "dev": dev, "flags": []}
             )
 
+    if exact:
+        routes = [route for route in routes if route.get("dst") == exact]
+
     if json_print:
         return json_dump(routes, pretty_json)
 
